<?php
/*
 * PV_SystemApp.php:
 * Web Application to visualize statistics of a photo voltaic system
 *
 * @author Joachim Schmidt - joachim.schmidt@jschmidt-systemberatung.de
 * @copyright Copyright (C) 2011 Joachim Schmidt. All rights reserved.
 * @license GNU/GPL
 *
 */
$text = array();

if (array_key_exists('lang', $_GET))
    $language = $_GET['lang'];
else
    $language = "de";

$request = $_GET['request'];
$report = $_GET['report'];
$type = $_GET['type'];
$month = $_GET['fmonth'];
$year = $_GET['fyear'];

$parms = array();
require_once 'pv_system_settings.php';

$parms['language'] = $language;
$start_senec = &$parms['start_senec'];
$kwp = &$parms['kwp'];
$price1_eeg = &$parms['price1_eeg'];
$price2_eeg = &$parms['price2_eeg'];
$percent_own_use = &$parms['percent_own_use'];
$own_use = $percent_own_use * 100;
$jos = &$parms['table_prefix'];
$price_evu = &$parms['price_evu']["year_" . $year];
$rgraph_library = &$parms['rgraph_library'];

$lib = $_SERVER['DOCUMENT_ROOT'] . $parms['lib'];
require_once ($lib);
$basepath = $_SERVER['DOCUMENT_ROOT'];
$parms['basepath'] = $basepath;

$language_ini = "PV_SystemAppText-" . $language . ".ini";
$text = parse_ini_file($language_ini, true);

$db_config = $_SERVER['DOCUMENT_ROOT'] . $parms['dbconfig'];

$root = $_SERVER['DOCUMENT_ROOT'];
$pv_system_root = &$parms['pv_system_root'];

// create chart object and set my chart properties
use phpRGraph\rgraph_chart;
use phpRGraph\rgraph_options;

$chart = new rgraph_chart("cvs", null, "Line");
$options = new rgraph_options("chart.ini");

$width = "680px;";
$height = "350px";
$div_width = "700px";
$div_height = "370px";
$responsive = array( 0.25,990,1200,$width,$height );

$canvas_style = "background-repeat: no-repeat; background-image: url(" . $pv_system_root . "/images/pv_system.jpg); background-size: 100% 100%;";

if ($language == "en")
{
    $options->set_option("scalePoint", ".", true);
    $options->set_option("scaleThousand", ",", true);
}
$result = "-1";
$options->set_gradientColors();
$options->set_option("debug", false);

// set chart properties
if ($type == 1)
{
    switch ($report)
    {

        case "1":
            $time_frame = $text['monat'][$month] . "  $year";
            if ($request == "chart")
            {
                $draw_option = "draw()";
                $chart->set_chartType('Bar');
                $options->set_option("variant", "3d", true);
                $options->set_option("colorsSequential", false, true);
                $options->set_option("marginBottom", 100, true);
                $options->set_option("marginLeft", 60, true);

                $options->set_option('title', $text['yield'] . " " . $time_frame, true);
                $sql = "select DATE_FORMAT(date, '" . $text['date_format'] . "'), Stromerzeugung,Netzeinspeisung from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31') order by date";

                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {
                    $labels = $result[0];
                    $options->set_option('labels', $labels, true);

                    $x = 15;
                    while (count($labels) < $x)
                    {
                        array_push($labels, " ");
                        array_push($result[1], null);
                        array_push($result[2], null);
                    }
                    
                    $chart->set_data(array(
                            $result[1],
                            $result[2]
                    ));
                    $chart->rearrangeData();
                  
                    if (max($result[2]) > 0)
                    {
                        $options->set_option("key", array(
                                $text['ownuse'],
                                $text['power_out']
                        ));
                        $options->set_option("keyPositionY", 5, true);
                        $options->set_option("keyInteractive", true, true);
                       
                        for ($i = 0; $i < count($result[0]); $i ++)
                            $result[1][$i] = $result[1][$i] - $result[2][$i];

                        $chart->set_data(array(
                                $result[1],
                                $result[2]
                        ));
                        $chart->rearrangeData();
                        $options->set_tooltip($chart->get_data(), "<b>#label#</b>: <br>#key# #val# kWh<br>(" . $text['type1'] . " #total# kWh)");
                        
                    }
                    else
                        $options->set_tooltip($chart->get_data(), $text['tooltip1']);

                    $options->set_option('textAngle', 30, true);

                    if (count($result[0]) > 15)
                    {
                        $labels = $options->xlabelsteps($labels, "2");
                        $options->set_option("marginInner", 4);
                    }
                    
                    $options->set_option("xaxisLabels", $labels, true);

                    $options->set_option('yaxisScaleUnitsPost', " kWh", true);
                    $options->set_option("grouping", "stacked", true);
                    $draw_option .= $options->getResponsiveFunction($responsive);
                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    $data = $text['errortitle'];
                    $data = $text['no_data'];
                    $data .= sprintf($text['error_nodata'], $time_frame);
                    echo $data;
                }
            }
            elseif ($request == "html")
            {
                // misc data:
                $html_out = "";

                // misc data cont:

                $sql = "select sum(Stromerzeugung) from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31')";
                $sum = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 3);
                if ($sum != - 1)
                {
                    $data['id'] = "html_text";
                    $data['data_type'] = "html_data";

                    $kwkwp = $sum / $kwp;
                    $sum1 = numberformat($sum, $language, 1);
                    $kwkwp = numberformat($kwkwp, $language, 1);

                    $sql = "select avg(Stromerzeugung) from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31')";
                    $avg = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 1);
                    $avg = numberformat($avg, $language, 1);
                    $html_out .= "<p style='font-size: 12px;'><b>" . $text['yield3'] . " $time_frame:  </b> " . $sum1 . " kWh, $kwkwp kWh " . $text['per'] . " kWp (" . $text['average'] . " " . $avg . " kWh " . $text['per_day'] . ")</p>";

                    $ertrag1 = $sum * $price1_eeg;
                    $ertrag1 = numberformat($ertrag1, $language, 2);

                    $sql = "select sum(Netzeinspeisung) from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31')";
                    $sum2 = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 1);
                    if ($sum2 > 0)
                        $percent_own_use = ($sum - $sum2) / $sum;
                    $ertrag2 = ($sum * (1 - $percent_own_use) * $price1_eeg) + ($sum * $percent_own_use * ($price2_eeg + $price_evu));
                    $ertrag2 = numberformat($ertrag2, $language, 2);
                    $used = round($sum * $percent_own_use);

                    $html_out .= $text['title4'] . $time_frame . "</h4>";
                    $html_out .= " <table width='100%'>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px; padding-top: 5px;'>" . $text['ertrag_1'] . "</td><td style='font-size: 12px;padding-top: 5px;'><b> $ertrag1 &euro; </b></td></tr>";
                    $html_out .= " <tr></tr>";
                    $html_out .= "<tr><td style='font-size: 12px; padding-left: 5px;padding-bottom: 5px;'>" . sprintf($text['ertrag_2'], $percent_own_use * 100) . " ($used kWh)</b>, " . $text['ertrag_3'] . " (" . numberformat($price_evu, $language, 4) . "&euro; " . $text['per'] .
                            " kWh): </td><td style='font-size: 12px; padding-left: 5px; padding-bottom: 5px; vertical-align:bottom;'><b> $ertrag2 &euro; </b></td></tr>";
                    $html_out .= " </table>";
                    $html_out .= $text['hint_solar'];

                    echo $html_out;
                }
                else
                {
                    $data = sprintf($text['error_nodata'], $time_frame);
                    echo $data;
                }
            }
            break;

        case "2":
            $time_frame = $text['year'] . " " . $year;
            if ($request == "chart")
            {
                $draw_option = "grow()";
                $chart->set_chartType('Bar');
                $options->set_option("variant", "3d");
                $options->set_option("marginLeft", 80);
                $options->set_option("marginBottom", 80);
                $options->set_option("variantThreedAngle", 0.05);
                $options->set_option("tooltipsEvent", "onmousemove");
                $options->set_option('yaxisScaleUnitsPost', " kWh");
                $options->set_option('scaleDecimals', 0);
                $options->set_option('yaxisScaleDecimals', 0);
                $options->set_option("colorsSequential", false, true);

                $options->set_option("highlightFill", $options->setOpacity("#fff", 0.4));

                $options->set_option('title', $text['yield'] . " " . $time_frame);
                // create the data portion via sql
                if ($language == "en")
                    $sql = "select DATE_FORMAT(date,'%M'),Stromerzeugung,Netzeinspeisung from  " . $jos . "pv_senec_year where date like '$year-%' order by date asc";
                else
                    $sql = "select month,Stromerzeugung,Netzeinspeisung from  " . $jos . "pv_senec_year where date like '$year-%' order by date asc";

                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {

                    $colors = $options->colors;
                    // var_dump($colors);
                    for ($i = 0; $i < count($result[0]); $i ++)
                    {
                        if ($result[1][$i] == 0)
                            $colors[$i] = "rgba(255,255,255,0)";
                    }
                    $options->set_option('colors', $colors);

                    $max = max($result[1]);
                    if ($max < 450)
                        $max = ceil($max / 10) * 10;
                    else
                        $max = ceil($max / 100) * 100;
                    
                    $options->set_option('yaxisScaleMax', $max);
                    $options->set_option('xaxisLabels', $result[0]);

                    $labels = $result[0];
                    $options->set_option('labels', $labels, true);

                    if (max($result[2]) > 0 && $year >= $parms['start_senec'] )
                    {
                        $options->set_option("key", array(
                                $text['ownuse'],
                                $text['power_out']
                        ));
                        $options->set_option("keyPositionY", 5, true);
                        $options->set_option("keyInteractive", true, true);
                          
                        for ($i = 0; $i < count($result[0]); $i ++)
                            $result[1][$i] = $result[1][$i] - $result[2][$i];

                        $chart->set_data(array(
                                $result[1],
                                $result[2]
                        ));
                        $chart->rearrangeData();
                        
                        $options->set_tooltip($chart->get_data(), "<b>#label#:</b> <br>#key# #val# kWh<br>(" . $text['type1'] . " #total# kWh)");
                    }
                    else
                    {
                        $chart->set_data(array(
                                $result[1]
                         ));
                        $chart->rearrangeData();
                        
                        $options->set_tooltip($chart->get_data(), "<b>" . $text['type1'] . " #label#</b><br> #val# KWh");
                        $colors = $options->get_colors();
                        $j = 0;

                        for ($i = count($colors) - 1; $i < 24; $i ++)
                        {
                            $colors[$i] = $colors[$j];
                            $j ++;
                        }

                        // print_r($colors);
                        $options->set_option('colors', $colors);
                        $options->set_option("colorsSequential", true, true);
                    }

                    $options->set_option("grouping", "stacked", true);

                    $draw_option .= $options->getResponsiveFunction($responsive);

                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    echo sprintf($text['error_nodata'], $time_frame);
                }
            }
            elseif ($request == "html")
            {
                $sql = "select sum(Stromerzeugung) from  " . $jos . "pv_senec_year where date like '$year%'";
                $sum = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 1);

                if ($sum != "-1")
                {
                    $ertrag1 = $sum * $price1_eeg;
                    $ertrag1 = numberformat($ertrag1, $language, 2);

                    $sql = "select sum(Netzeinspeisung) from  " . $jos . "pv_senec_year where date like '$year%'";
                    $sum2 = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 1);
                    if ($year > 2020 && $sum > 0)
                    {
                        $percent_own_use = ($sum - $sum2) / $sum;
                        $own_use = numberformat($percent_own_use * 100, $language, 0);
                    }

                    $ertrag2 = ($sum * (1 - $percent_own_use) * $price1_eeg) + ($sum * $percent_own_use * ($price2_eeg + $price_evu));
                    $ertrag2 = numberformat($ertrag2, $language, 2);
                    $used = round($sum * $percent_own_use);
                    $used = numberformat($used, $language, 0);
                    $kwkwp = $sum / $kwp;
                    $kwkwp = numberformat($kwkwp, $language, 1);
                    $sum = numberformat($sum, $language, 0);

                    $html_out = "<p style='font-size: 12px;'><b>" . $text['yield3'] . " $time_frame:</b> " . $sum . " kWh ($kwkwp kWh / kWp)</p>";
                    $html_out .= $text['title3'] . " $time_frame </h4>";
                    $html_out .= "<table width='100%'>";
                    $html_out .= "<tr><td width='65%' style='font-size: 12px;padding-top: 5px;'>" . $text['ertrag_1'] . "</td><td style='font-size: 12px;padding-top: 5px;'><b> $ertrag1 &euro; </b></td></tr>";
                    $html_out .= "<tr></tr>";
                    $html_out .= "<tr><td style='font-size: 12px; padding-bottom: 5px;'>" . sprintf($text['ertrag_2'], $own_use) . " ($used kWh)</b>, " . $text['ertrag_3'] . " (" . numberformat($price_evu, $language, 4) . "&euro; " . $text['per'] .
                            " kWh): </td><td style='font-size: 12px; padding-bottom: 5px; vertical-align:bottom;'><b> $ertrag2 &euro; </b></td></tr>";
                    $html_out .= "</table>";
                    if ($text['hints'])
                        $html_out .= $text['hints_day'];
                    echo $html_out;
                }
            }
            break;

        case "3":
            $time_frame = $text['totals'];
            if ($request == "chart")
            {
                $ymin = 000;
                $draw_option = "wave()";
                $chart->set_chartType('Bar');
                $options->set_option("variant", "3d");
                // $options->set_option("ymin", $ymin);
                $options->set_option('scaleDecimals', 0);
                $options->set_option('yaxisScaleDecimals', 0);
                $options->set_option("marginLeft", 90);
                $options->set_option("marginBottom", 100);
                // $options->set_option("marginTop", 60);
                $options->set_option("xaxisLabelsAngle", 20);
                $options->set_option("tooltipsEvent", "onmousemove");
                $options->set_option('yaxisScaleUnitsPost', " kWh");
                $options->set_option("highlightFill", $options->setOpacity("#fff", 0.4));
                $options->set_option('title', $text['yield5'] . " " . $time_frame);
                // create the data portion via sql
                $sql = "SELECT YEAR( date ), concat(SUM(Stromerzeugung)) AS summe FROM " . $jos . "pv_senec_year  GROUP BY YEAR( date )";
                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {
                    $colors = $options->colors;
                    for ($i = 0; $i < count($result[0]); $i ++)
                    {
                        if ($result[1][$i] == 0)
                        {
                            $result[1][$i] = $ymin;
                            $colors[$i] = "rgba(255,255,255,0)";
                        }
                    }
                    $options->set_option('colors', $colors);
                    $options->set_option('labels', $result[0]);
                    $options->set_option('xaxisLabels', $result[0]);
                    $chart->set_data($result[1]);
                    $options->set_tooltip($result[1], "<b>#label#:</b><br/>#val# kWh");
                    $draw_option .= $options->getResponsiveFunction($responsive);
                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    $data = sprintf($text['error_nodata'], $time_frame);
                    echo ($data);
                }
            }
            elseif ($request == "html")
            {
                $sql = "select sum(Stromerzeugung) from  " . $jos . "pv_senec_year";
                $sum = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 1);
                if ($sum != "-1")
                {
                    $data['id'] = "html_text";
                    $data['data_type'] = "html_data";
                    $ertrag1 = $sum * $price1_eeg;
                    $ertrag1 = numberformat($ertrag1, $language, 2);

                    $ertrag2 = calc_total_ertrag($parms);
                    $ertrag2 = numberformat($ertrag2, $language, 2);
                    $used = round($sum * $percent_own_use);
                    $used = numberformat($used, $language, 0);
                    $kwkwp = $sum / $kwp;
                    $kwkwp = numberformat($kwkwp, $language, 1);
                    $sum = numberformat($sum, $language, 0);

                    $html_out = "<p style='font-size: 12px;'><b>" . $text['yield4'] . " $time_frame:</b> " . $sum . " kWh ($kwkwp kWh / kWp)</p>";
                    $html_out .= $text['title4'] . " $time_frame </h4>";
                    $html_out .= "<table width='100%'>";
                    $html_out .= "<tr><td width='65%' style='font-size: 12px; padding-top: 5px;'>" . $text['ertrag_1'] . "</td><td style='font-size: 12px;padding-top: 5px;'><b> $ertrag1 &euro; </b></td></tr>";
                    $html_out .= "<tr></tr>";
                    $html_out .= "<tr><td style='font-size: 12px;padding-bottom: 5px;'>" . sprintf($text['ertrag_2'], $own_use) . " ($used kWh)</b>, " . rtrim($text['ertrag_3']) . ":</td><td style='font-size: 12px; padding-bottom: 5px;vertical-align:bottom;'><b> $ertrag2 &euro; </b></td></tr>";
                    $html_out .= "</table>";
                    if ($text['hints'])
                        $html_out .= $text['hints'];
                    echo $html_out;
                }
            }
            break;

        default:
            $data = "<b>bInvalid Request</b><br />";
            $data .= "Your request: &quot;$request&quot; is invalid";
            echo ($data);
    } // end switch
}

elseif ($type == 2 && $year . "-" . $month >= $start_senec)
{
    switch ($report)
    {

        case "1":
            $time_frame = $text['monat'][$month] . "  $year";
            if ($request == "chart")
            {
                $chart->set_chartType('Line');
                $draw_option = "trace({frames: 20})";
                $options->set_option('yaxisScaleUnitsPost', " kWh", true);
                $options->set_option('yaxisScaleDecimals', 0);
                $options->set_option('title', $text['type2'] . " - " . $time_frame, true);
                $options->set_option('linewidth', 1.5, true);
                $options->set_option("marginBottom", 60);
                $options->set_option("spline", true);
                $options->set_option("filled", false);
                $options->set_option("filledAccumulative", false);
                // $options->set_option("colors",
                // array("rgba(65,105,225,0.8)","rgba(0,102,0,0.6)","rgba(0,191,15,0.4)"
                // ));
                $sql = "select DATE_FORMAT(date, '" . $text['date_format'] . "'), Stromverbrauch,Netzbezug from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31') order by date";
                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {
                    $chart->set_data(array(
                            $result[1],
                            $result[2]
                    ));

                    $options->set_option("keyInteractive", true);
                    $options->set_option("key", array(
                            $text["power_use"],
                            $text["power_in"]
                    ), true);

                    $max = max($result[1]);
                    $max = ceil($max / 10) * 10;
                    $options->set_option('yaxisScaleMax', $max);
                    
                    $labels = $result[0];
                    $options->set_option('labels', $labels, true);
                    $options->set_tooltip($chart->get_data(), $text['tooltip4'], true);
                    if (count($result[0]) > 15)
                        $labels = $options->xlabelsteps($labels, "2");
                    $options->set_option("xaxisLabels", $labels, true);

                    $options->set_option('linewidth', 3.5, true);
                    $options->set_option('tickmarksStyle', 'circle');
                    $options->set_option('tickmarksSize', 2.0);
                    
                    $options->set_option('trendline', array(true));
                    $options->set_option('trendlineColors', array('#4169E1','rgba(0,102,0,0.5)'));
                    $options->set_option('trendlineLinewidth', array(3));
                    $options->set_option('trendlineDashed', true);
                    

                    $options->set_option("backgroundGridAutofit", false, true);
                    $draw_option .= $options->getResponsiveFunction($responsive);
                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    $data = $text['errortitle'];
                    $data .= $text['no_data'];
                    $data .= sprintf($text['error_nodata'], $time_frame);
                    $data .= $text['hint_nodata'];
                    echo ($data);
                }
            }
            elseif ($request == "html")
            {
                // misc data:
                $html_out = "";
                $sql = "select sum(Stromverbrauch),sum(Netzbezug),sum(Netzeinspeisung),sum(Stromerzeugung) from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31')";
                $result = rgraph_chart::getDataFromDB($db_config, $sql);
                if ($result != "-1")
                {
                    $cost_1 = round($result[0][0] * $price_evu, 2);
                    $cost_2 = round($result[1][0] * $price_evu, 2);
                    $saved_1 = round($result[2][0] * $price1_eeg, 2);
                    $own_use = round($result[3][0] - $result[2][0], 2);
                    $own_use_percent = $own_use * 100 / $result[3][0];
                    $html_out = "<p style='font-size: 12px;'>" . $text['own_use'] . " - " . $time_frame . ": </b>" . numberformat($own_use, $language, 2) . "kWh (=" . round($own_use_percent) . "%)";

                    $ertrag2 = ($own_use * ($price2_eeg + $price_evu));
                    $bilanz = $saved_1  - $cost_2;
                    $bilanz1 = $saved_1 + $ertrag2 - $cost_2;
                    //$bilanz2 = numberformat($cost_1 + $bilanz1, $language, 2);
                    $ertrag2 = numberformat($ertrag2, $language, 2);
                    $cost_1 = numberformat($cost_1, $language, 2);
                    $cost_2 = numberformat($cost_2, $language, 2);
                    $saved_1 = numberformat($saved_1, $language, 2);

                    $html_out .= $text['title5'] . $time_frame . "</h4>";
                    $html_out .= " <table width='100%'>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px; padding-top: 5px;'>" . $text['kosten_1'] . " " . round($result[0][0], 0) . " kWh)</td><td class='td_number' style='padding-top: 5px;'><b> $cost_1 &euro; </b></td></tr>";
                    $html_out .= " <tr><td>&nbsp;</td><td>&nbsp;</td></tr>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;'>" . $text['kosten_2'] . "(" . round($result[1][0], 0) . " kWh)</td><td class='td_number'><b> $cost_2 &euro; </b></td></tr>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;'>" . $text['saving_1'] . "(" . round($result[2][0], 0) . " kWh)</td><td class='td_number'><b> $saved_1 &euro; </b></td></tr>";
                    $html_out .= " <tr><td style='font-size: 12px; padding-left: 5px;padding-bottom: 5px;'>&nbsp;" . $text['own_use'] . " (" . numberformat($own_use, $language, 0) . "kWh)</b>, " . $text['ertrag_3'] . " (" . numberformat($price_evu, $language, 4) . "&euro; " . $text['per'] .
                            " kWh): </td><td class='td_number' style='padding-bottom: 5px; vertical-align:bottom;'><b> $ertrag2 &euro; </b></td></tr>";
                    $html_out .= " <tr></tr>";
                    if ($bilanz < 0)
                    {
                        $bilanz_text = $text['bilanz_2'];
                        $bilanz = $bilanz * -1;
                        $bilanz = "<span style='color: red'>" . numberformat($bilanz, $language, 2) . "</span>";
                    }
                    else
                    {
                        $bilanz_text = $text['bilanz_1'];
                        $bilanz = numberformat($bilanz, $language, 2);
                    }
                    if ($bilanz1 < 0)
                    {
                        $gewinn_text = $text['verlust'];
                        $bilanz1 = $bilanz1 * -1;
                        $bilanz1 = "<span style='color: red'>" . numberformat($bilanz1, $language, 2) . "</span>";
                    }
                    else
                    {
                        $gewinn_text = $text['gewinn'];
                        $bilanz1 = numberformat($bilanz1, $language, 2);
                    }
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;padding-top: 5px;'>" . $bilanz_text . "</td><td class='td_number' style='padding-top: 5px;'><b> $bilanz &euro; </b></td></tr>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;padding-bottom: 5px'>" . $gewinn_text . "</td><td class='td_number'><b> $bilanz1 &euro; </b></td></tr>";
                    $html_out .= " </table>";
                    $html_out .= $text['hint_panel'];
                }

                else
                {
                    $html_out .= $text['hint_solar'];
                }
                echo $html_out;
            }
            break;

        case "2":
            $time_frame = "$year";
            if ($request == "chart")
            {
                $chart->set_chartType('Bar');
                $draw_option = "draw()";
                $options->set_option('yaxisScaleUnitsPost', " kWh", true);
                $options->set_option('yaxisScaleDecimals', 0);
                $options->set_option('title', $text['type2'] . " - " . $time_frame, true);
                $options->set_option('marginInner', 4);
                $options->set_option("marginInnerGrouped", 0);
                $options->set_option("marginBottom", 100);
                $options->set_option("colorsSequential", false);
                $options->set_option("grouping", "grouped");
                $options->set_option("variant", "3d");

                if ($language == "en")
                    $sql = "select DATE_FORMAT(date,'%M'),Stromverbrauch,Netzbezug from  " . $jos . "pv_senec_year where date like '$year-%' order by date asc";
                else
                    $sql = "select month,Stromverbrauch,Netzbezug,Netzeinspeisung from  " . $jos . "pv_senec_year where date like '$year-%' order by date asc";

                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {
                    $labels = $result[0];
                    $options->set_option('labels', $labels, true);

                    $chart->set_data(array(
                            $result[1],
                            $result[2]
                    ));
                    $chart->rearrangeData();

                    $options->set_option("key", array(
                            $text["power_use"],
                            $text["power_in"]
                    ));
                    $options->set_option("keyPositionY", 5, true);
                    $options->set_option("keyInteractive", true, true);
                    $options->set_tooltip($chart->get_data(), "<b>#label#:</b><br> #key# #val# KWh");

                    $options->set_option('textAngle', 30, true);

                    $labels = $result[0];
                    $options->set_option('labels', $labels, true);
                    
                    if (count($result[0]) > 15)
                        $labels = $options->xlabelsteps($labels, "2");

                    $options->set_option("xaxisLabels", $labels, true);
                    $options->set_option("backgroundGridAutofit", false, true);
                    $draw_option .= $options->getResponsiveFunction($responsive);
                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    $data = $text['errortitle'];
                    $data .= $text['no_data'];
                    $data .= sprintf($text['error_nodata'], $time_frame);
                    $data .= $text['hint_nodata'];
                    echo ($data);
                }
            }
            elseif ($request == "html")
            {
                // misc data:
                $html_out = "";
                $sql = "select sum(Stromverbrauch),sum(Netzbezug),sum(Netzeinspeisung),sum(Stromerzeugung) from  " . $jos . "pv_senec_year where date like '$year-%'";
                $result = rgraph_chart::getDataFromDB($db_config, $sql);
                if ($result != "-1")
                {
                    $cost_1 = round($result[0][0] * $price_evu, 2);
                    $cost_2 = round($result[1][0] * $price_evu, 2);
                    $saved_1 = round($result[2][0] * $price1_eeg, 2);
                    $own_use = round($result[3][0] - $result[2][0], 2);
                    $own_use_percent = $own_use * 100 / $result[3][0];
                    $html_out = "<p style='font-size: 12px;'>" . $text['own_use'] . " - " . $time_frame . ": </b>" . numberformat($own_use, $language, 2) . "kWh (=" . round($own_use_percent) . "%)";

                    $ertrag2 = ($own_use * ($price2_eeg + $price_evu));
                    $bilanz = $saved_1 - $cost_2;
                    $bilanz1 = $saved_1 + $ertrag2 - $cost_2;
                    //$bilanz2 = numberformat($cost_1 + $bilanz1, $language, 2);
                    $ertrag2 = numberformat($ertrag2, $language, 2);
                    $cost_1 = numberformat($cost_1, $language, 2);
                    $cost_2 = numberformat($cost_2, $language, 2);
                    $saved_1 = numberformat($saved_1, $language, 2);

                    $html_out .= $text['title5'] . $time_frame . "</h4>";
                    $html_out .= " <table width='100%'>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px; padding-top: 5px;'>" . $text['kosten_1'] . " " . round($result[0][0], 0) . " kWh)</td><td class='td_number' style='padding-top: 5px;'><b> $cost_1 &euro; </b></td></tr>";
                    $html_out .= " <tr><td>&nbsp;</td><td>&nbsp;</td></tr>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;'>" . $text['kosten_2'] . "(" . round($result[1][0], 0) . " kWh)</td><td class='td_number'><b> $cost_2 &euro; </b></td></tr>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;'>" . $text['saving_1'] . "(" . round($result[2][0], 0) . " kWh)</td><td class='td_number'><b> $saved_1 &euro; </b></td></tr>";
                    $html_out .= " <tr><td style='font-size: 12px; padding-left: 5px;padding-bottom: 5px;'>&nbsp;" . $text['own_use'] . " (" . numberformat($own_use, $language, 0) . "kWh)</b>, " . $text['ertrag_3'] . " (" . numberformat($price_evu, $language, 4) . "&euro; " . $text['per'] .
                            " kWh): </td><td class='td_number' style='padding-bottom: 5px; vertical-align:bottom;'><b> $ertrag2 &euro; </b></td></tr>";
                    $html_out .= " <tr></tr>";
                    if ($bilanz < 0)
                    {
                        $bilanz_text = $text['bilanz_2'];
                        $bilanz = $bilanz * -1;
                        $bilanz = "<span style='color: red'>" . numberformat($bilanz, $language, 2) . "</span>";
                    }
                    else
                    {
                        $bilanz_text = $text['bilanz_1'];
                        $bilanz = numberformat($bilanz, $language, 2);
                    }
                    if ($bilanz1 < 0)
                    {
                        $gewinn_text = $text['verlust'];
                        $bilanz1 = $bilanz1 * -1;
                        $bilanz1 = "<span style='color: red'>" . numberformat($bilanz1, $language, 2) . "</span>";
                    }
                    else
                    {
                        $gewinn_text = $text['gewinn'];
                        $bilanz1 = numberformat($bilanz1, $language, 2);
                    }
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;padding-top: 5px;'>" . $bilanz_text . "</td><td class='td_number' style='padding-top: 5px;'><b> $bilanz &euro; </b></td></tr>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px;padding-bottom: 5px'>" . $gewinn_text . "</td><td class='td_number'><b> $bilanz1 &euro; </b></td></tr>";

                    $html_out .= " </table>";
                    $html_out .= $text['hints'];
                }

                else
                {
                    $html_out .= $text['hint_solar'];
                }
                echo $html_out;
            }
            break;

        case "3":
            $time_frame = "<b>&quot;" . $text['totals'] . "&quot;</b>";
            $data = $text['errortitle'];
            $data .= $text['no_data'];
            $data .= sprintf($text['error_nodata'], $time_frame);
            echo ($data);
            break;

        default:
            $data = "<b>bInvalid Request</b><br />";
            $data .= "Your request: &quot;$request&quot; is invalid";
            echo ($data);
    } // end switch
}

elseif ($type == 3 && $year . "-" . $month >= $start_senec)
{
    switch ($report)
    {
        case "1":
            $time_frame = $text['monat'][$month] . "  $year";
            if ($year == 2020 and $month == 12)
                $min_day = "$year-$month-08";
            else
                $min_day = "$year-$month-01";
            if ($request == "chart")
            {
                $chart->set_chartType('Line');
                $draw_option = "trace({frames: 20})";
                $options->set_option('yaxisScaleUnitsPost', " %", true);
                $options->set_option('yaxisScaleDecimals', 0);
                $options->set_option('title', $text['type3'] . " - " . $time_frame, true);
                $options->set_option('linewidth', 2.5, true);
                $options->set_option("marginBottom", 60);
                $options->set_option("spline", true);
                $options->set_option("yaxisScaleMax", 100.1);
                $options->set_option("filled", false);
                $options->set_option("filledAccumulative", false);
                $sql = "select DATE_FORMAT(date, '" . $text['date_format'] . "'), Stromverbrauch,Netzbezug,Netzeinspeisung,Stromerzeugung from  " . $jos . "pv_senec_days where (date>='$min_day' and date<='$year-$month-31') order by date";
                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {
                    $autark = array();
                    for ($i = 0; $i < count($result[0]); $i ++)
                    {
                        if ($result[1][$i] == 0)
                            $autark[$i] = 0;
                        else
                        {
                            // echo "<br>" . $result[1][$i] . " - "
                            // .$result[2][$i];
                            $autark[$i] = (100 * ($result[1][$i] - $result[2][$i])) / $result[1][$i];
                            if ($autark[$i] < 0)
                                $autark[$i] = 0;
                        }
                    }

                    $own_use = array();

                    for ($i = 0; $i < count($result[4]); $i ++)
                    {
                        if ($result[4][$i] != 0)
                            $own_use[$i] = (100 * ($result[4][$i] - $result[3][$i])) / $result[4][$i];
                    }

                    $chart->set_data(array(
                            $autark,
                            $own_use
                    ));

                    $options->set_option("keyInteractive", true);
                    $options->set_option("key", array(
                            $text["autark"],
                            $text["ownuse"]
                    ), true);
                    $options->set_option("keyPositionY", 150, true);

                    $labels = $result[0];
                    $options->set_option('labels', $labels, true);
                    $options->set_tooltip($chart->get_data(), $text['tooltip5'], true);
                    if (count($result[0]) > 15)
                        $labels = $options->xlabelsteps($labels, "2");
                    $options->set_option("xaxisLabels", $labels, true);

                    $options->set_option('linewidth', 3.0, true);
                    $options->set_option('tickmarksStyle', 'circle');
                    $options->set_option('tickmarksSize', 2.5);

                    $options->set_option("backgroundGridAutofit", false, true);
                    $draw_option .= $options->getResponsiveFunction($responsive);
                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    $data = $text['errortitle'];
                    $data .= $text['no_data'];
                    $data .= sprintf($text['error_nodata'], $time_frame);
                    $data .= $text['hint_nodata'];
                    echo ($data);
                }
            }
            elseif ($request == "html")
            {
                // misc data:
                $html_out = "";
                $sql = "select sum(Stromverbrauch),sum(Netzbezug),sum(Netzeinspeisung),sum(Stromerzeugung) from  " . $jos . "pv_senec_days where (date>='$year-$month-01' and date<='$year-$month-31')";
                $result = rgraph_chart::getDataFromDB($db_config, $sql);
                if ($result != "-1")
                {
                    $own_use = round($result[3][0] - $result[2][0], 2);
                    $own_use_percent = $own_use * 100 / $result[3][0];
                    $html_out = "<p style='font-size: 12px;'>" . $text['own_use'] . " - " . $time_frame . ": </b>" . numberformat($own_use, $language, 2) . "kWh (=" . round($own_use_percent) . "%)";

                    $autark = numberformat(100 * ($result[0][0] - $result[1][0]) / $result[0][0], $language, 1);

                    $html_out .= $text['title5'] . $time_frame . "</h4>";
                    $html_out .= " <table width='100%'>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px; padding-top: 5px;padding-bottom: 5px;'>" . $text['autark_1'] . " </td><td style='font-size: 12px;padding-top: 5px;'><b> $autark % </b></td></tr>";

                    $html_out .= " </table>";
                    $html_out .= $text['hint_battery'];
                }

                else
                {
                    $html_out .= $text['hint_solar'];
                }
                echo $html_out;
            }
            break;

        case "2":
            if ($year < 2021)
                $year = 2021;
            $time_frame = $year;
            if ($request == "chart")
            {
 
                $chart->set_chartType('Line');
                $draw_option = "trace({frames: 20})";
                $options->set_option('yaxisScaleUnitsPost', " %", true);
                $options->set_option('yaxisScaleDecimals', 0);
                $options->set_option('title', $text['type3'] . " - " . $time_frame, true);
                $options->set_option('linewidth', 2.5, true);
                $options->set_option("marginBottom", 60);
                $options->set_option("spline", true);
                $options->set_option("yaxisScaleMax", 100.1);
                $options->set_option("filled", false);
                $options->set_option("filledAccumulative", false);

                if ($language == "en")
                    $sql = "select DATE_FORMAT(date,'%M'),Stromverbrauch,Netzbezug,Netzeinspeisung,Stromerzeugung from  " . $jos . "pv_senec_year where date like '$year-%' order by date asc";
                else
                    $sql = "select month,Stromverbrauch,Netzbezug,Netzeinspeisung,Stromerzeugung from  " . $jos . "pv_senec_year where date like '$year-%' order by date asc";

                $result = rgraph_chart::getDataFromDB($db_config, $sql);

                if ($result != "-1")
                {
                    for ($i = 0; $i < count($result[0]); $i ++)
                        if ($result[1][$i] != 0)
                            $labels[$i] = $result[0][$i];
                    
                    $autark = array();
                    for ($i = 0; $i < count($labels); $i ++)
                    {
                        if ($result[1][$i] == 0)
                            $autark[$i] = null;
                        else
                            $autark[$i] = (100 * ($result[1][$i] - $result[2][$i])) / $result[1][$i];
                    }

                    $own_use = array();
                    for ($i = 0; $i < count($labels); $i ++)
                    {
                        if ($result[4][$i] == 0)
                            $own_use[$i] = null;
                        else
                            $own_use[$i] = (100 * ($result[4][$i] - $result[3][$i])) / $result[4][$i];
                    }

                    $chart->set_data(array(
                            $autark,
                            $own_use
                    ));
 
                    $options->set_option("keyInteractive", true);
                    $options->set_option("key", array(
                            $text["autark"],
                            $text["ownuse"]
                    ), true);

                    $options->set_option('labels', $labels, true);
                    $options->set_tooltip($chart->get_data(), $text['tooltip6'], true);
                    $options->set_option("xaxisLabels", $labels, true);
                        
                    $options->set_option('linewidth', 3.0, true);
                    $options->set_option('tickmarksStyle', 'circle');
                    $options->set_option('tickmarksSize', 2.5);

                    $options->set_option("backgroundGridAutofit", false, true);
                    $draw_option .= $options->getResponsiveFunction($responsive);
                    $chart->set_options($options);
                    $rgraph_json = $chart->toString();
                    include $root . $parms['pv_system_root'] . "/default.php";
                }
                else
                {
                    $data = $text['errortitle'];
                    $data .= $text['no_data'];
                    $data .= sprintf($text['error_nodata'], $time_frame);
                    $data .= $text['hint_nodata'];
                    echo ($data);
                }
            }
            elseif ($request == "html")
            {
                // misc data:
                $html_out = "";

                $sql = "select sum(Stromverbrauch),sum(Netzbezug),sum(Netzeinspeisung),sum(Stromerzeugung) from  " . $jos . "pv_senec_year where date like '$year-%'";

                $result = rgraph_chart::getDataFromDB($db_config, $sql);
                if ($result != "-1")
                {
                    $own_use = round($result[3][0] - $result[2][0], 2);
                    $own_use_percent = $own_use * 100 / $result[3][0];
                    $html_out = "<p style='font-size: 12px;'>" . $text['own_use'] . " - " . $time_frame . ": </b>" . numberformat($own_use, $language, 2) . "kWh (=" . round($own_use_percent) . "%)";

                    $autark = numberformat(100 * ($result[0][0] - $result[1][0]) / $result[0][0], $language, 1);

                    $html_out .= $text['title5'] . $time_frame . "</h4>";
                    $html_out .= " <table width='100%'>";
                    $html_out .= " <tr><td width='65%' style='font-size: 12px; padding-left: 5px; padding-top: 5px;padding-bottom: 5px;'>" . $text['autark_1'] . " </td><td style='font-size: 12px;padding-top: 5px;'><b> $autark % </b></td></tr>";

                    $html_out .= " </table>";
                    $html_out .= $text['hint_senec'];
                }

                else
                {
                    $html_out .= $text['hint_senec'];
                }
                echo $html_out;
            }

            break;

        case "3":
            $time_frame = "<b>&quot;" . $text['totals'] . "&quot;</b>";
            $data = $text['errortitle'];
            $data .= $text['no_data'];
            $data .= sprintf($text['error_nodata'], $time_frame);
            echo ($data);
            break;

        default:
            $data = "<b>bInvalid Request</b><br />";
            $data .= "Your request: &quot;$request&quot; is invalid";
            echo ($data);
    }
}

else
{
    $data = $text['error'];
    $data .= " &quot;" . $text['monat'][$month] . " $year&quot; ";
    echo ($data);
}

exit();

function convert_type ($input)
{
    $output = array();
    for ($i = 0; $i < count($input); $i ++)
    {
        $output[$i] = (float) $input[$i];
    }
    return $output;
}

function numberformat ($number, $language, $precision)
{
    if ($language == "de")
        return number_format($number, $precision, ',', '.');
    else
        return number_format($number, $precision, '.', ',');
}

function calc_total_ertrag ($parms)
{

    // get properties from parms
    $price1_eeg = &$parms['price1_eeg'];
    $price2_eeg = &$parms['price2_eeg'];
    $percent_own_use = &$parms['percent_own_use'];
    $jos = &$parms['table_prefix'];

    $db_config = &$parms['dbconfig'];
    $db_config = $_SERVER['DOCUMENT_ROOT'] . $db_config;

    $sql = "SELECT YEAR( date ) FROM " . $jos . "pv_senec_year where Stromerzeugung > 0 GROUP BY YEAR( date )";
    $years = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, null);
    $years = explode(",", $years);
    $ertrag = 0;

    for ($i = 0; $i < count($years); $i ++)
    {
        $sql = "select sum(Stromerzeugung) from  " . $jos . "pv_senec_year where date like '" . $years[$i] . "%'";
        $price_evu = &$parms['price_evu']["year_" . $years[$i]];
        $sum = PV_SystemApp_ProcessSQLRequest($db_config, $sql, null, 1);
        $ertrag = $ertrag + ($sum * (1 - $percent_own_use) * $price1_eeg) + ($sum * $percent_own_use * ($price2_eeg + $price_evu));
    }
    return $ertrag;
}

function PV_SystemApp_ProcessSQLRequest ($db_config, $sql, $type = NULL, $precision = 3, $factor = 1)
{
    require_once ($db_config);

    $jconfig = new JConfig();
    $db = $jconfig->db;
    $dbuser = $jconfig->user;
    $dbpw = $jconfig->password;

    if ($sql != '')
        $query = $sql;
    else
        return "-1";

    $connection = mysqli_connect("localhost", $dbuser, $dbpw, $db);
    if ($connection == null)
    {
        echo "could not connect to database $db";
        return "-1";
    }

    mysqli_query($connection, "set character set utf8");
    $result = mysqli_query($connection, $query);

    if ($result == false)
    {
        echo "no result object via sql $sql";
        return "-1";
    }

    $data = "";
    $label = null;
    $cols = mysqli_num_fields($result);

    if (mysqli_num_rows($result) == 0)
        return "-1";

    $data_cols = array();
    for ($i = 0; $i < $cols; $i ++)
        $data_cols[$i] = "";

    $row = mysqli_fetch_row($result);
    while ($row)
    {
        for ($i = 0; $i < $cols; $i ++)
        {
            if (is_numeric($row[$i]))
            {
                if ($factor != 1)
                    $row[$i] *= $factor;
                $data_cols[$i] .= round($row[$i], $precision) . ",";
            }
            else
                $data_cols[$i] .= $row[$i] . ",";
        }
        $row = mysqli_fetch_row($result);
    }

    for ($i = 0; $i < $cols; $i ++)
        $data_cols[$i] = rtrim($data_cols[$i], ",");

    if ($cols > 1)
    {
        $label = $data_cols[0] . "/";
        for ($i = 1; $i < $cols; $i ++)
            $data .= $data_cols[$i] . "|";
        $data = rtrim($data, "|");
    }
    elseif ($cols == 1)
        $data = $data_cols[0];

    mysqli_free_result($result);

    if ($data == "")
        return "-1";
    if ($cols > 1)
        $data = $label . $data;

    return $data;
}

?>
     


